@@ -53,6 +53,7 @@ MS_Usage()
     echo "    --lz4              : Compress using lz4 instead of gzip"
     echo "    --compress         : Compress using the UNIX 'compress' command"
     echo "    --complevel lvl    : Compression level for gzip pigz zstd xz lzo lz4 bzip2 pbzip2 and bzip3 (default 9)"
+    echo "    --comp-extra       : Append extra options to the chosen compressor"
     echo "    --threads thds     : Number of threads to be used by compressors that support parallelization."
     echo "                         Omit to use compressor's default. Most useful (and required) for opting"
     echo "                         into xz's threading, usually with '--threads=0' for all available cores."
@@ -93,6 +94,7 @@ MS_Usage()
     echo "    --nocrc            : Don't calculate a CRC for archive"
     echo "    --sha256           : Compute a SHA256 checksum for the archive"
     echo "    --header file      : Specify location of the header script"
+    echo "    --preextract file  : Specify a pre-extraction script"
     echo "    --cleanup file     : Specify a cleanup script that executes on interrupt and when finished successfully."
     echo "    --follow           : Follow the symlinks in the archive"
     echo "    --noprogress       : Do not show the progress during the decompression"
@@ -110,6 +112,10 @@ MS_Usage()
     echo "    --keep-umask       : Keep the umask set to shell default, rather than overriding when executing self-extracting archive."
     echo "    --export-conf      : Export configuration variables to startup_script"
     echo
+    echo "ENVIRONMENT"
+    echo "    SETUP_NOCHECK"
+    echo "        If set to 1, then checksum validation will be skipped."
+    echo
     echo "Do not forget to give a fully qualified startup script name"
     echo "(i.e. with a ./ prefix if inside the archive)."
     exit 1
@@ -129,6 +135,7 @@ PASSWD=""
 PASSWD_SRC=""
 OPENSSL_NO_MD=n
 COMPRESS_LEVEL=9
+COMP_EXTRA=""
 DEFAULT_THREADS=123456 # Sentinel value
 THREADS=$DEFAULT_THREADS
 KEEP=n
@@ -248,6 +255,10 @@ do
 	COMPRESS_LEVEL="$2"
     shift 2 || { MS_Usage; exit 1; }
 	;;
+    --comp-extra)
+	COMP_EXTRA="$2"
+    shift 2 || { MS_Usage; exit 1; }
+	;;
     --threads)
 	THREADS="$2"
     shift 2 || { MS_Usage; exit 1; }
@@ -307,6 +318,12 @@ do
 	HEADER="$2"
     shift 2 || { MS_Usage; exit 1; }
 	;;
+    --preextract)
+    preextract_file="$2"
+    shift 2 || { MS_Usage; exit 1; }
+    test -r "$preextract_file" || { echo "Unable to open pre-extraction script: $preextract_file" >&2; exit 1; }
+    PREEXTRACT_ENCODED=`base64 < "$preextract_file"`
+    ;;
     --cleanup)
     CLEANUP_SCRIPT="$2"
     shift 2 || { MS_Usage; exit 1; }
@@ -537,13 +554,17 @@ none)
     ;;
 esac
 
+if test x"$COMP_EXTRA" != "x"; then
+    GZIP_CMD="$GZIP_CMD $COMP_EXTRA"
+fi
+
 if test x"$ENCRYPT" = x"openssl"; then
     if test x"$APPEND" = x"y"; then
         echo "Appending to existing archive is not compatible with OpenSSL encryption." >&2
     fi
     
-    ENCRYPT_CMD="openssl enc -aes-256-cbc -salt"
-    DECRYPT_CMD="openssl enc -aes-256-cbc -d"
+    ENCRYPT_CMD="openssl enc -aes-256-cbc -salt -pbkdf2"
+    DECRYPT_CMD="openssl enc -aes-256-cbc -d -salt -pbkdf2"
     
     if test x"$OPENSSL_NO_MD" != x"y"; then
         ENCRYPT_CMD="$ENCRYPT_CMD -md sha256"
